<?php
class ome_mdl_api_log_elk extends dbeav_model{
    public $filter_use_like = true;
    private $keyMapping = array(
        'method' => 'method',
        'step' => 'api_type',
        'title' => 'task_name',
    );
    private $key = "erp_jst";
    private $secret = "kjdf923hfkovjh023jr";
    public function table_name($real=false){
        $tableName = 'api_log';
        return $real ? kernel::database()->prefix.'ome_'.$tableName : $tableName;
    }
    public function get_schema(){
        $schema = app::get('ome')->model('api_log')->get_schema();
        return $schema;
    }

    function gen_id(){
        return uniqid();
    }


    /*
     * 写日志
     * @param int $log_id 日志id
     * @param string $task_name 操作名称
     * @param string $class 调用这次api请求方法的类
     * @param string $method 调用这次api请求方法的类函数
     * @param array $params 调用这次api请求方法的参数集合
     * @param string $msg 返回信息
     * @param string $addon[marking_value标识值，marking_type标识类型 ]
     *
     */
    function write_log($log_id,$task_name,$class,$method,$params,$memo='',$api_type='request',$status='running',$msg='',$addon='',$log_type='',$bn=''){
        $time = time();

        if (isset($params[1]['msg_id'])) {
            $msg_id = $params[1]['msg_id'];
        }
        $log_sdf = array(
            'log_id' => $log_id,
            'task_name' => $task_name,
            'status' => $status,
            'worker' => $class.':'.$method,
            'params' => serialize($params),
            'msg' => $msg,
            'log_type' => $log_type,
            'api_type' => $api_type,
            'memo' => $memo,
            'original_bn' => $bn,
            'createtime' => $time,
            'last_modified' => $time,
        );
        if ($msg_id) {
            $log_sdf['msg_id'] = $msg_id;
        }
        if (is_array($addon)){
            $log_sdf['marking_value'] = $addon['marking_value'];
            $log_sdf['marking_type'] = $addon['marking_type'];
        }

        return $this->insert($log_sdf);
    }

    function update_log($log_id,$msg=NULL,$status=NULL,$params=NULL,$addon=NULL){

    }

    function retry($log_id='', $retry_type='', $isSelectedAll='', $cursor='0'){
        $filter = array(
            'status' => 'fail',
            'step' => 'request'
        );
        if ($retry_type=='batch' and ( strstr($log_id,"|") or $isSelectedAll == '_ALL_' ) ){
            //批量重试
            $limit = 1;
            if ($isSelectedAll != '_ALL_'){
                $log_ids = explode('|',$log_id);
                $filter['log_id'] = $log_ids[$cursor];
                $lim = 0;
            }else{
                $lim = $cursor * $limit;
            }
            $row = $this->getList('*', $filter, $lim, $limit, ' createtime asc ');
            if ($row){
                foreach ($row as $k=>$v){
                    $detail = $this->dump(array('log_id' => $v['log_id'], 'status' => 'fail'));

                    if (!$detail) continue;

                    return $this->start_api_retry($detail);
                }

                return array('task_name'=>'全部批量重试', 'status'=>'complete');
            }else{
                return array('task_name'=>'全部批量重试', 'status'=>'complete');
            }
        }else{
            //单个按钮重试
            $filter['log_id'] = $log_id;
            $row = $this->dump($filter);
            return $this->start_api_retry($row);
        }
    }

    /*
     * 发起API同步重试
     * @param array $row 发起重试数据
     */
    function start_api_retry($row){
        return array('task_name' => $row['task_name'], 'status' => 'fail');
    }

    public function insert(&$params) {
        if($params['data']) {
            $data = $params['data'];
        } else {
            $data = array('result' => $params['msg']);
            $arr = unserialize($params['params']);
            if($arr) {
                if(!$params['method']) {
                    $params['method'] = $arr[0];
                }
                $data['params'] = $arr;
            }
        }
        $kafkaData = array(
            'title'       => (string) $params['task_name'],
            'method'      => (string) $params['method'],
            'original_bn' => (string) $params['original_bn'],
            'msg_id'      => (string) $params['msg_id'],
            'status'      => $params['status'],
            'createtime'  => intval($params['createtime']),
            'spendtime'   => $params['spendtime'] ? $params['spendtime'] : 0,
            'data'        => $data
        );
        kernel::single('erpapi_log_elk')->write_log($kafkaData, $params['api_type']);
    }

    public function update($data,$filter,$mustUpdate = null) {
        return true;
    }

    public function _filter($filter,$tableAlias=null,$baseWhere=null) {
        $filtered = array(
            // 'filter' => array(
            //     'bool' => array(
            //         'must' => array(
            //             array('term'=>array('node_id'=>base_shopnode::node_id('ome') )),
            //         ),
            //     ),
            // ),
           'query' => array(
                'bool' => array(
                    'must' => array(
                        array('match'=>array('node_id'=>base_shopnode::node_id('ome') )),
                    ),
                ),
            ),
        );

        $directField = array(
            'msg_id'      => 'msg_id',
            // 'original_bn' => 'original_bn',
            'status'      => 'status',
            'step'        => 'step',
            'log_id'      => '_id',
            'api_method'  => 'method',
            'api_type'    => 'step',
        );
        foreach($filter as $k => $val) {
            if(isset($directField[$k])) {
                $term = array('match'=>array($directField[$k]=>$val));
                array_push($filtered['query']['bool']['must'],$term); unset($filter[$k]);
            }
        }

        if ($filter['task_name']) {
            $filtered['query']['bool']['must'][] = array(
                'match' => array(
                    'title' => array('query' => $filter['task_name'], "minimum_should_match"=> "100%")
                ),
            );

            unset($filter['task_name']);
        }

        if ($filter['original_bn']) {
            $filtered['query']['bool']['must'][] = array(
                'match' => array(
                    'original_bn' => array('query' => $filter['original_bn'], "minimum_should_match"=> "100%")
                ),
            );

            unset($filter['original_bn']);
        }

        if ($filter['params']) {
            $filtered['query']['bool']['must'][] = array(
                'regexp' => array(
                    'data' => "({$filter['params']})"
                ),
            );

            unset($filter['params']);
        }


        if($filter['createtime_from']) {
            $createTime['range']['createtime']['gte'] = strtotime($filter['createtime_from']);
        }
        if($filter['createtime_to']) {
            $createTime['range']['createtime']['lt'] = strtotime($filter['createtime_to'] . ' 23:59:59');
        }

        if ($createTime) $filtered['filter']['bool']['must'][] = $createTime;

        return array('filtered' => $filtered);
    }

    public function count($filter=null){
        if(empty($filter)) {
            return ' ';
        }
        $url = ELK_APILOG_URL;
        $filter = $this->_filter($filter);
        $params = array(
            'fields' => array('original_bn'),
            'query' => $filter,
            'size' => 1,
            'from' => 0
        );
        $data = $this->elk_search($url, $params);
        return $data['hits'] ? $data['hits']['total'] > 9000 ? 9000 : $data['hits']['total'] : ' ';
    }

    public function getList($cols='*', $filter=array(), $offset=0, $limit=-1, $orderType=null){
         if(count($filter) < 3) {
            return array();
        }
        $filter = $this->_filter($filter);
        $params = array(
            'fields' => array('createtime', 'method', 'msg_id', 'original_bn', 'status', 'step', 'title', 'type'),
            'query' => $filter,
            'sort' => array('@timestamp' => array('order' => 'desc'))
        );
        $url = ELK_APILOG_URL;
        if($limit != -1) {
            $params['size'] = $limit;
            $params['from'] = $offset;
        }
        $data = $this->elk_search($url, $params);
        if($data && $data['hits'] && $data['hits']['total']) {
            return $this->_formatElkData($data['hits']['hits']);
        } else {
            return array();
        }
    }

    public function dump($filter,$field = '*',$subSdf = null) {
        if(is_array($filter)) {
            $filter = $this->_filter($filter);
        } else {
            $filter = $this->_filter(array('log_id'=>$filter));
        }
        $params = array(
            'fields' => array('spendtime', 'title', 'method', 'original_bn', 'msg_id', 'status', 'createtime', 'step', 'node_id', 'domain', 'type', 'data'),
            'query' => $filter,
            'size' => 1,
            'from' => 0
        );
        $url = ELK_APILOG_URL;
        $data = $this->elk_search($url, $params);
        $rs = array();
        if($data['hits']['hits'][0]) {
            $rows = $this->_formatElkData($data['hits']['hits']);
            $rs = $this->_formatRowElkData($rows[0]);
        }
        return $rs;
    }
    
    function is_repeat($key=''){
        $paramsCacheLib = kernel::single('taoexlib_params_cache');
        $paramsCacheLib->fetch($key, $value);
        return $value;
    }

    function set_repeat($key='',$log_id=''){
        $paramsCacheLib = kernel::single('taoexlib_params_cache');
        return $paramsCacheLib->store($key, $log_id,$expries_time=1200);
    }

    private function elk_search($url, $params) {
        $now = time();
        $sdf = array(
            'auth' => array(
                'key' => $this->key,
                'value' => md5("n" . $this->secret . "e" . $now . "0"),
                'number' => $now,
            ),
            'query' => $params
        );
        $rs = kernel::single('base_httpclient')->post($url, json_encode($sdf));
        return json_decode($rs, true);
    }

    private function _formatElkData($data) {
        $arrData = array();
        foreach($data as $row) {
            $tmp = array();
            foreach($row['fields'] as $k => $val) {
                $index = $this->keyMapping[$k] ? $this->keyMapping[$k] : $k;
                if($k == 'createtime') {
                    $pos = strpos($val[0], '.');
                    if($pos) {
                        $tmp[$index] = substr($val[0], 0, $pos);
                    } else {
                        $tmp[$index] = $val[0];
                    }
                } else {
                    $tmp[$index] = $val[0];
                }
            }
            $tmp['log_id'] = $row['_id'];
            $tmp['retry'] = 0;
            $tmp['last_modified'] = $tmp['createtime'];
            $arrData[] = $tmp;
        }
        return $arrData;
    }

    private function _formatRowElkData($row) {
        $tmp = array();
        $msg = array();
        $tmp[0] = $row['method'];
        $allData = json_decode($row['data'], true);
        foreach ($allData as $k => $val) {
            if(in_array($k, array('result', 'response'))) {
                $msg = $val;
            } elseif($k == 'callback') {
                continue;
            } elseif($val) {
                $tmp[1][$k] = var_export($val, true) . '<br />';
            }
        }
        $row['params'] = serialize($tmp);
        if(is_array($msg)) {
            $row['msg'] = json_encode($msg);
        } else {
            $row['msg'] = $msg;
        }
        return $row;
    }
}
